@@ -213,7 +213,7 @@ anjuta_vcs_status_get_type (void)
 {
   static GType etype = 0;
   if (etype == 0) {
-    static const GEnumValue values[] = {
+    static const GFlagsValue values[] = {
       { ANJUTA_VCS_STATUS_MODIFIED, "ANJUTA_VCS_STATUS_MODIFIED", "modified" },
       { ANJUTA_VCS_STATUS_ADDED, "ANJUTA_VCS_STATUS_ADDED", "added" },
       { ANJUTA_VCS_STATUS_DELETED, "ANJUTA_VCS_STATUS_DELETED", "deleted" },
@@ -225,7 +225,7 @@ anjuta_vcs_status_get_type (void)
       { ANJUTA_VCS_STATUS_IGNORED, "ANJUTA_VCS_STATUS_IGNORED", "ignored" },
       { 0, NULL, NULL }
     };
-    etype = g_enum_register_static (g_intern_static_string ("AnjutaVcsStatus"), values);
+    etype = g_flags_register_static (g_intern_static_string ("AnjutaVcsStatus"), values);
   }
   return etype;
 }
