@@ -55,8 +55,6 @@
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
 #define FILE_BUFFER_SIZE 1024
 
 static void
@@ -361,7 +359,8 @@ anjuta_util_dialog_error_system (GtkWindow* parent, gint errnum,
 	va_end (args);
 
 	if (0 != errnum) {
-		tot_mesg = g_strconcat (message, _("\nSystem: "),
+		/* Avoid space in translated string */
+		tot_mesg = g_strconcat (message, "\n", _("System:"), " ",
 								g_strerror(errnum), NULL);
 		g_free (message);
 	} else
@@ -1937,3 +1936,56 @@ anjuta_util_get_user_data_file_path (const gchar* path, ...)
 	return file_path;
 }
 
+GList *
+anjuta_util_convert_gfile_list_to_path_list (GList *list)
+{
+	GList *path_list;
+	GList *current_file;
+	gchar *path;
+	
+	path_list = NULL;
+	current_file = list;
+	
+	while (current_file)
+	{
+		path = g_file_get_path (current_file->data);
+		
+		/* Ignore files with invalid paths */
+		if (path)
+			path_list = g_list_append (path_list, path);
+	}
+	
+	return path_list;
+}
+
+GList *
+anjuta_util_convert_gfile_list_to_relative_path_list (GList *list, 
+                                                      const gchar *parent)
+{
+	GFile *parent_file;
+	GList *path_list;
+	GList *current_file;
+	gchar *path;
+	
+	parent_file = g_file_new_for_path (parent);
+	path_list = NULL;
+	
+	if (parent_file)
+	{
+		current_file = list;
+		
+		while (current_file)
+		{
+			path = g_file_get_relative_path (parent_file, current_file->data);
+			
+			/* Ignore files with invalid paths */
+			if (path)
+				path_list = g_list_append (path_list, path);
+		}
+		
+		g_object_unref (parent_file);
+	}
+	
+	return path_list;
+}
+
