@@ -36,15 +36,15 @@ typedef enum
 {
 	/* Unversioned, ignored, or uninteresting items */
 	ANJUTA_VCS_STATUS_NONE = 0, /*< skip >*/
-	ANJUTA_VCS_STATUS_MODIFIED,
-	ANJUTA_VCS_STATUS_ADDED,
-	ANJUTA_VCS_STATUS_DELETED,
-	ANJUTA_VCS_STATUS_CONFLICTED,
-	ANJUTA_VCS_STATUS_UPTODATE,
-	ANJUTA_VCS_STATUS_LOCKED,
-	ANJUTA_VCS_STATUS_MISSING,
-	ANJUTA_VCS_STATUS_UNVERSIONED,
-	ANJUTA_VCS_STATUS_IGNORED
+	ANJUTA_VCS_STATUS_MODIFIED = 1 << 0,
+	ANJUTA_VCS_STATUS_ADDED = 1 << 1,
+	ANJUTA_VCS_STATUS_DELETED = 1 << 2,
+	ANJUTA_VCS_STATUS_CONFLICTED = 1 << 3,
+	ANJUTA_VCS_STATUS_UPTODATE = 1 << 4,
+	ANJUTA_VCS_STATUS_LOCKED = 1 << 5,
+	ANJUTA_VCS_STATUS_MISSING = 1 << 6,
+	ANJUTA_VCS_STATUS_UNVERSIONED = 1 << 7,
+	ANJUTA_VCS_STATUS_IGNORED = 1 << 8
 } AnjutaVcsStatus;
 
 #endif // ANJUTA_VCS_STATUS_H
