@@ -171,7 +171,6 @@ get_vcs_emblem (AnjutaVcsStatus status)
 		default:
 			emblem = NULL;
 	}
-	DEBUG_PRINT ("emblem found: %d", emblem != NULL);
 	return emblem;
 }
 
@@ -212,8 +211,11 @@ file_model_vcs_status_callback(GFile *file,
 									-1);
 				if (file_icon)
 				{
+					GdkPixbuf *new_icon;
+
+					new_icon = gdk_pixbuf_copy (file_icon);
 					gdk_pixbuf_composite (emblem,
-										  file_icon,
+										  new_icon,
 										  0, 0,
 										  gdk_pixbuf_get_width (file_icon),
 										  gdk_pixbuf_get_height (file_icon),
@@ -224,10 +226,10 @@ file_model_vcs_status_callback(GFile *file,
 					gtk_tree_store_set (GTK_TREE_STORE (model),
 										&iter,
 										COLUMN_PIXBUF,
-										file_icon,
+										new_icon,
 										-1);
 					DEBUG_PRINT ("%s", "setting emblem");
-					
+					g_object_unref (new_icon);
 					g_object_unref (file_icon);
 				}
 				g_object_unref (emblem);
@@ -259,13 +261,7 @@ file_model_get_vcs_status (FileModel* model,
 	GtkTreePath* path = gtk_tree_model_get_path (GTK_TREE_MODEL(model),
 												 iter);
 	FileModelPrivate* priv = FILE_MODEL_GET_PRIVATE(model);	
-	AnjutaPlugin* plugin = ANJUTA_PLUGIN(g_object_get_data (G_OBJECT(priv->view), "__plugin"));
-	
-	g_return_if_fail (plugin != NULL);
-
-	IAnjutaVcs* ivcs = anjuta_shell_get_interface (plugin->shell,
-												   IAnjutaVcs,
-												   NULL);
+	IAnjutaVcs* ivcs = g_object_get_data (G_OBJECT(priv->view), "__ivcs");
 	if (ivcs)
 	{	
 		VcsData* data = g_new0(VcsData, 1);
