@@ -43,7 +43,7 @@
 #define ANJUTA_PIXMAP_COMPLETE			  "anjuta-complete"
 #define ANJUTA_PIXMAP_AUTOCOMPLETE        "anjuta-complete-auto"
 #define ANJUTA_PIXMAP_AUTOINDENT          "anjuta-indent-auto"
-#define ANJUTA_STOCK_SWAP                     "anjuta-swap"
+#define ANJUTA_STOCK_SWAP                 "anjuta-swap"
 #define ANJUTA_STOCK_COMPLETE         	  "anjuta-complete"
 #define ANJUTA_STOCK_AUTOCOMPLETE         "anjuta-autocomplete"
 #define ANJUTA_STOCK_AUTOINDENT           "anjuta-indent"
@@ -139,7 +139,7 @@ skip_iter_to_newline_tail (IAnjutaIterable *iter, gchar ch)
 /* Jumps to the reverse matching brace of the given brace character */
 
 static gint
-get_line_indentation (IAnjutaEditor *editor, gint line_num)
+get_line_indentation (IAnjutaEditor *editor, gint line_num, gint* line_indent_spaces)
 {
 	IAnjutaIterable *line_begin, *line_end;
 	gchar *line_string, *idx;
@@ -176,7 +176,7 @@ get_line_indentation (IAnjutaEditor *editor, gint line_num)
 		if (*idx == '\t')
 			line_indent += TAB_SIZE;
 		else
-			line_indent++;
+			line_indent_spaces++;
 		idx++; /* Since we are looking for first non-space char, simple
 	            * increment of the utf8 chars would do */
 	}
@@ -185,35 +185,32 @@ get_line_indentation (IAnjutaEditor *editor, gint line_num)
 }
 
 static gchar *
-get_line_indentation_string (IAnjutaEditor *editor, gint spaces)
+get_line_indentation_string (IAnjutaEditor *editor, gint spaces, gint line_indent_spaces)
 {
 	gint i;
-	
-	/* DEBUG_PRINT ("In %s()", __FUNCTION__); */
-	g_return_val_if_fail (spaces >= 0, NULL);
-	
-	if (spaces <= 0)
+	gchar *indent_string;
+		
+	if ((spaces + line_indent_spaces) <= 0)
 		return NULL;
 	
 	if (USE_SPACES_FOR_INDENTATION)
 	{
-		gchar *indent_string = g_new0 (gchar, spaces + 1);
-		for (i = 0; i < spaces; i++)
+		indent_string = g_new0 (gchar, spaces + line_indent_spaces + 1);
+		for (i = 0; i < (spaces + line_indent_spaces); i++)
 			indent_string[i] = ' ';
-		return indent_string;
 	}
 	else
 	{
 		gint num_tabs = spaces / TAB_SIZE;
 		gint num_spaces = spaces % TAB_SIZE;
-		gchar *indent_string = g_new0 (gchar, num_tabs + num_spaces + 1);
+		indent_string = g_new0 (gchar, num_tabs + num_spaces + line_indent_spaces + 1);
 		
 		for (i = 0; i < num_tabs; i++)
 			indent_string[i] = '\t';
-		for (; i < num_tabs + num_spaces; i++)
+		for (; i < num_tabs + (num_spaces + line_indent_spaces); i++)
 			indent_string[i] = ' ';
-		return indent_string;
 	}
+	return indent_string;
 }
 
 /* Sets the iter to line end of previous line and TRUE is returned.
@@ -675,7 +672,7 @@ initialize_indentation_params (CppJavaPlugin *plugin)
 }
 
 static gint
-set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
+set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation, gint line_indent_spaces)
 {
 	IAnjutaIterable *line_begin, *line_end, *indent_position;
 	IAnjutaIterable *current_pos;
@@ -721,10 +718,10 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 	//DEBUG_PRINT ("carat offset is = %d", carat_offset);
 	
 	/* Set new indentation */
-	if (indentation > 0)
+	if ((indentation + line_indent_spaces) > 0)
 	{
-		indent_string = get_line_indentation_string (editor, indentation);
-		nchars = g_utf8_strlen (indent_string, -1);
+		indent_string = get_line_indentation_string (editor, indentation, line_indent_spaces);
+		nchars = indent_string ? g_utf8_strlen (indent_string, -1) : 0;
 		
 		/* Only indent if there is something to indent with */
 		if (indent_string)
@@ -822,6 +819,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 						   IAnjutaEditor *editor,
 						   gint line_num,
 						   gint *incomplete_statement,
+						   gint *line_indent_spaces,
 						   gboolean *colon_indent)
 {
 	IAnjutaIterable *iter;
@@ -834,6 +832,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	gboolean line_checked_for_comment = FALSE;
 	
 	*incomplete_statement = -1;
+	*line_indent_spaces = 0;
 	
 	if (line_num <= 1)
 		return 0;
@@ -843,7 +842,8 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	iter = ianjuta_editor_get_line_begin_position (editor, line_num, NULL);
 	
 	current_line_is_preprocessor = line_is_preprocessor (editor, iter);
-	current_line_is_continuation = line_is_continuation (editor, iter);
+	current_line_is_continuation = 
+		line_is_continuation (editor, iter);
 	/*
 	DEBUG_PRINT ("Current line is preprocessor = %d",
 				 current_line_is_preprocessor);
@@ -856,7 +856,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	{
 		/* Continuation of preprocessor line -- just maintain indentation */
 		g_object_unref (iter);
-		return get_line_indentation (editor, line_num - 1);;
+		return get_line_indentation (editor, line_num - 1, line_indent_spaces);
 	}
 	else if (current_line_is_preprocessor)
 	{
@@ -1030,7 +1030,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			/* If at level 0 indentation, encoutered a
 			 * block end, don't bother going further
 			 */
-			if (point_ch == '}' && get_line_indentation (editor, line_saved) <= 0)
+			if (point_ch == '}' && get_line_indentation (editor, line_saved, line_indent_spaces) <= 0)
 			{
 				line_indent = 0;
 				line_indent += extra_indent;
@@ -1040,16 +1040,16 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			/* Find matching brace and continue */
 			if (!cpp_java_util_jump_to_matching_brace (iter, point_ch, -1))
 			{
-				line_indent = get_line_indentation (editor, line_saved);
+				line_indent = get_line_indentation (editor, line_saved, line_indent_spaces);
 				line_indent += extra_indent;
 				break;
 			}
-			}
+		}
 		else if (point_ch == '{')
 		{
 			gint line_for_indent =
 				ianjuta_editor_get_line_from_position (editor, iter, NULL);
-			line_indent = get_line_indentation (editor, line_for_indent);
+			line_indent = get_line_indentation (editor, line_for_indent, line_indent_spaces);
 			/* Increase line indentation */
 			line_indent += INDENT_SIZE;
 			line_indent += extra_indent;
@@ -1077,9 +1077,9 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 				if (dummy_ch == '\t')
 					line_indent += TAB_SIZE;
 				else
-					line_indent++;
+					(*line_indent_spaces)++;
 			}
-			line_indent += 1;
+			(*line_indent_spaces)++;
 			line_indent += extra_indent;
 			
 			/* Although statement is incomplete at this point, we don't
@@ -1175,7 +1175,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 		line_indent += extra_indent;
 	}
 	g_object_unref (iter);
-	
+		
 	return line_indent;
 }
 
@@ -1220,7 +1220,7 @@ is_iter_inside_string (IAnjutaIterable *iter)
 
 static gint
 get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
-						   gint line)
+						   gint line, gint *line_indent_spaces)
 {
 	IAnjutaIterable *iter;
 	IAnjutaIterable *end_iter;
@@ -1237,12 +1237,14 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 	
 	if (is_iter_inside_string (iter))
 	{
-		line_indent = get_line_indentation (editor, line - 1);
+		line_indent = get_line_indentation (editor, line - 1, line_indent_spaces);
 	}
 	else
 	{
 		line_indent = get_line_indentation_base (plugin, editor, line,
-												 &incomplete_statement, &colon_indent);
+												 &incomplete_statement, 
+												 line_indent_spaces,
+												 &colon_indent);
 	}
 	
 	if (colon_indent)
@@ -1277,7 +1279,7 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 		 */
 		if (is_iter_inside_string (iter))
 		{
-			line_indent = get_line_indentation (editor, line - 1);
+			line_indent = get_line_indentation (editor, line - 1, line_indent_spaces);
 			break;
 		}
 		ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (iter),
@@ -1317,13 +1319,14 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 				gint line = ianjuta_editor_get_line_from_position (editor,
 																   iter,
 																   NULL);
-				line_indent = get_line_indentation (editor, line);
+				line_indent = get_line_indentation (editor, line, line_indent_spaces);
 			}
 			break;
 		}
 		else if (ch == '#')
 		{
 			line_indent = 0;
+			*line_indent_spaces = 0;
 		}
 		else if (!isspace (ch))
 		{
@@ -1335,6 +1338,7 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 	}
 	while (ianjuta_iterable_next (iter, NULL));
 	g_object_unref (iter);
+	
 	return line_indent;
 }
 
@@ -1404,12 +1408,13 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 		{
 			gint insert_line;
 			gint line_indent;
+			gint line_indent_spaces;
 		
 			ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 			initialize_indentation_params (plugin);
 			insert_line = ianjuta_editor_get_lineno (editor, NULL);
-			line_indent = get_line_auto_indentation (plugin, editor, insert_line);
-			set_line_indentation (editor, insert_line, line_indent);
+			line_indent = get_line_auto_indentation (plugin, editor, insert_line, &line_indent_spaces);
+			set_line_indentation (editor, insert_line, line_indent, line_indent_spaces);
 			ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 		}
 	}
@@ -1825,10 +1830,12 @@ on_auto_indent (GtkAction *action, gpointer data)
 	
 	for (insert_line = line_start; insert_line <= line_end; insert_line++)
 	{
+		gint line_indent_spaces = 0;
 		line_indent = get_line_auto_indentation (lang_plugin, editor,
-												 insert_line);
+												 insert_line,
+												 &line_indent_spaces);
 		/* DEBUG_PRINT ("Line indent for line %d = %d", insert_line, line_indent); */
-		set_line_indentation (editor, insert_line, line_indent);
+		set_line_indentation (editor, insert_line, line_indent, line_indent_spaces);
 	}
 	ianjuta_document_end_undo_action (IANJUTA_DOCUMENT(editor), NULL);
 }
