@@ -74,6 +74,23 @@ on_reset_layout_activate(GtkAction *action, AnjutaApp *app)
 	anjuta_app_layout_reset (app);
 }
 
+void
+on_toolbar_view_toggled (GtkAction *action, AnjutaApp *app)
+{
+	gboolean status = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
+	if (status)
+	{
+		gtk_widget_show (app->toolbar);
+	}
+	else
+	{
+		gtk_widget_hide (app->toolbar);
+	}
+	anjuta_preferences_set_int (app->preferences,
+								"anjuta.toolbar.visible",
+								status);
+}
+
 void
 on_set_preferences1_activate (GtkAction * action, AnjutaApp *app)
 {
@@ -102,7 +119,7 @@ on_set_preferences1_activate (GtkAction * action, AnjutaApp *app)
 }
 
 static void
-help_activate (GtkWindow *parent, const gchar *doc_id, const gchar *item)
+help_activate (GtkWidget *parent, const gchar *doc_id, const gchar *item)
 {
 	anjuta_util_help_display (parent, doc_id, item);
 }
